@@ -83,8 +83,6 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define gst_shm_sink_parent_class parent_class
 G_DEFINE_TYPE (GstShmSink, gst_shm_sink, GST_TYPE_BASE_SINK);
-GST_ELEMENT_REGISTER_DEFINE (shmsink, "shmsink", GST_RANK_NONE,
-    GST_TYPE_SHM_SINK);
 
 static void gst_shm_sink_finalize (GObject * object);
 static void gst_shm_sink_set_property (GObject * object, guint prop_id,
@@ -354,7 +352,6 @@ gst_shm_sink_init (GstShmSink * self)
 {
   g_cond_init (&self->cond);
   self->size = DEFAULT_SIZE;
-  self->unlock = FALSE;
   self->wait_for_connection = DEFAULT_WAIT_FOR_CONNECTION;
   self->perms = DEFAULT_PERMS;
 
@@ -420,12 +417,12 @@ gst_shm_sink_class_init (GstShmSinkClass * klass)
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   signals[SIGNAL_CLIENT_CONNECTED] = g_signal_new ("client-connected",
-      GST_TYPE_SHM_SINK, G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-      G_TYPE_NONE, 1, G_TYPE_INT);
+      GST_TYPE_SHM_SINK, G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 
   signals[SIGNAL_CLIENT_DISCONNECTED] = g_signal_new ("client-disconnected",
-      GST_TYPE_SHM_SINK, G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-      G_TYPE_NONE, 1, G_TYPE_INT);
+      GST_TYPE_SHM_SINK, G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 
   gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
 
@@ -673,11 +670,6 @@ gst_shm_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   gsize written_bytes;
 
   GST_OBJECT_LOCK (self);
-  if (self->unlock) {
-    GST_OBJECT_UNLOCK (self);
-    return GST_FLOW_FLUSHING;
-  }
-
   while (self->wait_for_connection && !self->clients) {
     g_cond_wait (&self->cond, GST_OBJECT_GET_LOCK (self));
     if (self->unlock) {
